@@ -29,6 +29,8 @@ from upath.registry import get_upath_class
 
 from airflow.io.store import attach
 from airflow.io.utils.stat import stat_result
+from airflow.lineage.hook import get_hook_lineage_collector
+from airflow.utils.log.logging_mixin import LoggingMixin
 
 if typing.TYPE_CHECKING:
     from fsspec import AbstractFileSystem
@@ -39,6 +41,42 @@ PT = typing.TypeVar("PT", bound="ObjectStoragePath")
 default = "file"
 
 
+class TrackingFileWrapper(LoggingMixin):
+    """Wrapper that tracks file operations to intercept lineage."""
+
+    def __init__(self, path: ObjectStoragePath, obj):
+        super().__init__()
+        self._path: ObjectStoragePath = path
+        self._obj = obj
+
+    def __getattr__(self, name):
+        attr = getattr(self._obj, name)
+        if callable(attr):
+            # If the attribute is a method, wrap it in another method to intercept the call
+            def wrapper(*args, **kwargs):
+                self.log.debug("Calling method: %s", name)
+                if name == "read":
+                    get_hook_lineage_collector().add_input_dataset(context=self._path, uri=str(self._path))
+                elif name == "write":
+                    get_hook_lineage_collector().add_output_dataset(context=self._path, uri=str(self._path))
+                result = attr(*args, **kwargs)
+                return result
+
+            return wrapper
+        return attr
+
+    def __getitem__(self, key):
+        # Intercept item access
+        return self._obj[key]
+
+    def __enter__(self):
+        self._obj.__enter__()
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self._obj.__exit__(exc_type, exc_val, exc_tb)
+
+
 class ObjectStoragePath(CloudPath):
     """A path-like object for object storage."""
 
@@ -121,7 +159,7 @@ class ObjectStoragePath(CloudPath):
     def open(self, mode="r", **kwargs):
         """Open the file pointed to by this path."""
         kwargs.setdefault("block_size", kwargs.pop("buffering", None))
-        return self.fs.open(self.path, mode=mode, **kwargs)
+        return TrackingFileWrapper(self, self.fs.open(self.path, mode=mode, **kwargs))
 
     def stat(self) -> stat_result:  # type: ignore[override]
         """Call ``stat`` and return the result."""
@@ -160,7 +198,7 @@ class ObjectStoragePath(CloudPath):
 
         Returns the new Path instance pointing to the target path.
         """
-        return self.rename(target, overwrite=True)
+        return self.rename(target)
 
     @classmethod
     def cwd(cls):
@@ -188,7 +226,8 @@ class ObjectStoragePath(CloudPath):
         return self.fs.checksum(self.path)
 
     def read_block(self, offset: int, length: int, delimiter=None):
-        r"""Read a block of bytes.
+        r"""
+        Read a block of bytes.
 
         Starting at ``offset`` of the file, read ``length`` bytes. If
         ``delimiter`` is set then we ensure that the read starts and stops at
@@ -223,7 +262,8 @@ class ObjectStoragePath(CloudPath):
         return self.fs.read_block(self.path, offset=offset, length=length, delimiter=delimiter)
 
     def sign(self, expiration: int = 100, **kwargs):
-        """Create a signed URL representing the given path.
+        """
+        Create a signed URL representing the given path.
 
         Some implementations allow temporary URLs to be generated, as a
         way of delegating credentials.
@@ -259,7 +299,8 @@ class ObjectStoragePath(CloudPath):
             shutil.copyfileobj(f1, f2, **kwargs)
 
     def copy(self, dst: str | ObjectStoragePath, recursive: bool = False, **kwargs) -> None:
-        """Copy file(s) from this path to another location.
+        """
+        Copy file(s) from this path to another location.
 
         For remote to remote copies, the key used for the destination will be the same as the source.
         So that s3://src_bucket/foo/bar will be copied to gcs://dst_bucket/foo/bar and not
@@ -273,6 +314,11 @@ class ObjectStoragePath(CloudPath):
         if isinstance(dst, str):
             dst = ObjectStoragePath(dst)
 
+        if self.samestore(dst) or self.protocol == "file" or dst.protocol == "file":
+            # only emit this in "optimized" variants - else lineage will be captured by file writes/reads
+            get_hook_lineage_collector().add_input_dataset(context=self, uri=str(self))
+            get_hook_lineage_collector().add_output_dataset(context=dst, uri=str(dst))
+
         # same -> same
         if self.samestore(dst):
             self.fs.copy(self.path, dst.path, recursive=recursive, **kwargs)
@@ -316,14 +362,14 @@ class ObjectStoragePath(CloudPath):
                     continue
 
                 src_obj._cp_file(dst)
-
             return
 
         # remote file -> remote dir
         self._cp_file(dst, **kwargs)
 
     def move(self, path: str | ObjectStoragePath, recursive: bool = False, **kwargs) -> None:
-        """Move file(s) from this path to another location.
+        """
+        Move file(s) from this path to another location.
 
         :param path: Destination path
         :param recursive: bool
@@ -335,6 +381,8 @@ class ObjectStoragePath(CloudPath):
             path = ObjectStoragePath(path)
 
         if self.samestore(path):
+            get_hook_lineage_collector().add_input_dataset(context=self, uri=str(self))
+            get_hook_lineage_collector().add_output_dataset(context=path, uri=str(path))
             return self.fs.move(self.path, path.path, recursive=recursive, **kwargs)
 
         # non-local copy
