@@ -17,7 +17,7 @@
 # under the License.
 from __future__ import annotations
 
-from functools import cached_property
+from functools import cached_property, lru_cache
 from time import sleep
 from typing import TYPE_CHECKING, Callable, NoReturn
 
@@ -34,6 +34,7 @@ from airflow.listeners.listener import get_listener_manager
 from airflow.models.base import ID_LEN, Base
 from airflow.serialization.pydantic.job import JobPydantic
 from airflow.stats import Stats
+from airflow.traces.tracer import Trace, span
 from airflow.utils import timezone
 from airflow.utils.helpers import convert_camel_to_snake
 from airflow.utils.log.logging_mixin import LoggingMixin
@@ -49,6 +50,8 @@ if TYPE_CHECKING:
 
     from sqlalchemy.orm.session import Session
 
+    from airflow.executors.base_executor import BaseExecutor
+
 
 def _resolve_dagrun_model():
     from airflow.models.dagrun import DagRun
@@ -56,6 +59,19 @@ def _resolve_dagrun_model():
     return DagRun
 
 
+@lru_cache
+def health_check_threshold(job_type: str, heartrate: int) -> int | float:
+    grace_multiplier = 2.1
+    health_check_threshold_value: int | float
+    if job_type == "SchedulerJob":
+        health_check_threshold_value = conf.getint("scheduler", "scheduler_health_check_threshold")
+    elif job_type == "TriggererJob":
+        health_check_threshold_value = conf.getfloat("triggerer", "triggerer_health_check_threshold")
+    else:
+        health_check_threshold_value = heartrate * grace_multiplier
+    return health_check_threshold_value
+
+
 class Job(Base, LoggingMixin):
     """
     The ORM class representing Job stored in the database.
@@ -104,14 +120,16 @@ class Job(Base, LoggingMixin):
     Only makes sense for SchedulerJob and BackfillJob instances.
     """
 
-    def __init__(self, executor=None, heartrate=None, **kwargs):
+    def __init__(self, executor: BaseExecutor | None = None, heartrate=None, **kwargs):
         # Save init parameters as DB fields
         self.heartbeat_failed = False
         self.hostname = get_hostname()
         if executor:
             self.executor = executor
+            self.executors = [executor]
         self.start_date = timezone.utcnow()
         self.latest_heartbeat = timezone.utcnow()
+        self.previous_heartbeat = None
         if heartrate is not None:
             self.heartrate = heartrate
         self.unixname = getuser()
@@ -131,22 +149,18 @@ class Job(Base, LoggingMixin):
     def heartrate(self) -> float:
         return Job._heartrate(self.job_type)
 
-    def is_alive(self, grace_multiplier=2.1) -> bool:
+    def is_alive(self) -> bool:
         """
         Is this job currently alive.
 
         We define alive as in a state of RUNNING, and having sent a heartbeat
         within a multiple of the heartrate (default of 2.1)
-
-        :param grace_multiplier: multiplier of heartrate to require heart beat
-            within
         """
+        threshold_value = health_check_threshold(self.job_type, self.heartrate)
         return Job._is_alive(
-            job_type=self.job_type,
-            heartrate=self.heartrate,
             state=self.state,
+            health_check_threshold_value=threshold_value,
             latest_heartbeat=self.latest_heartbeat,
-            grace_multiplier=grace_multiplier,
         )
 
     @provide_session
@@ -186,48 +200,62 @@ class Job(Base, LoggingMixin):
         :param session to use for saving the job
         """
         previous_heartbeat = self.latest_heartbeat
+        with Trace.start_span(span_name="heartbeat", component="Job") as span:
+            try:
+                span.set_attribute("heartbeat", str(self.latest_heartbeat))
+                # This will cause it to load from the db
+                self._merge_from(Job._fetch_from_db(self, session))
+                previous_heartbeat = self.latest_heartbeat
 
-        try:
-            # This will cause it to load from the db
-            self._merge_from(Job._fetch_from_db(self, session))
-            previous_heartbeat = self.latest_heartbeat
+                if self.state == JobState.RESTARTING:
+                    self.kill()
 
-            if self.state == JobState.RESTARTING:
-                self.kill()
+                # Figure out how long to sleep for
+                sleep_for = 0
+                if self.latest_heartbeat:
+                    seconds_remaining = (
+                        self.heartrate - (timezone.utcnow() - self.latest_heartbeat).total_seconds()
+                    )
+                    sleep_for = max(0, seconds_remaining)
+                if span.is_recording():
+                    span.add_event(name="sleep", attributes={"sleep_for": sleep_for})
+                sleep(sleep_for)
 
-            # Figure out how long to sleep for
-            sleep_for = 0
-            if self.latest_heartbeat:
-                seconds_remaining = (
-                    self.heartrate - (timezone.utcnow() - self.latest_heartbeat).total_seconds()
-                )
-                sleep_for = max(0, seconds_remaining)
-            sleep(sleep_for)
+                job = Job._update_heartbeat(job=self, session=session)
+                self._merge_from(job)
+                time_since_last_heartbeat = (timezone.utcnow() - previous_heartbeat).total_seconds()
+                health_check_threshold_value = health_check_threshold(self.job_type, self.heartrate)
+                if time_since_last_heartbeat > health_check_threshold_value:
+                    self.log.info("Heartbeat recovered after %.2f seconds", time_since_last_heartbeat)
+                # At this point, the DB has updated.
+                previous_heartbeat = self.latest_heartbeat
 
-            job = Job._update_heartbeat(job=self, session=session)
-            self._merge_from(job)
-
-            # At this point, the DB has updated.
-            previous_heartbeat = self.latest_heartbeat
-
-            heartbeat_callback(session)
-            self.log.debug("[heartbeat]")
-        except OperationalError:
-            Stats.incr(convert_camel_to_snake(self.__class__.__name__) + "_heartbeat_failure", 1, 1)
-            if not self.heartbeat_failed:
-                self.log.exception("%s heartbeat got an exception", self.__class__.__name__)
-                self.heartbeat_failed = True
-            if self.is_alive():
-                self.log.error(
-                    "%s heartbeat failed with error. Scheduler may go into unhealthy state",
-                    self.__class__.__name__,
-                )
-            else:
-                self.log.error(
-                    "%s heartbeat failed with error. Scheduler is in unhealthy state", self.__class__.__name__
-                )
-            # We didn't manage to heartbeat, so make sure that the timestamp isn't updated
-            self.latest_heartbeat = previous_heartbeat
+                heartbeat_callback(session)
+                self.log.debug("[heartbeat]")
+                self.heartbeat_failed = False
+            except OperationalError:
+                Stats.incr(convert_camel_to_snake(self.__class__.__name__) + "_heartbeat_failure", 1, 1)
+                if not self.heartbeat_failed:
+                    self.log.exception("%s heartbeat failed with error", self.__class__.__name__)
+                    self.heartbeat_failed = True
+                    msg = f"{self.__class__.__name__} heartbeat got an exception"
+                    if span.is_recording():
+                        span.add_event(name="error", attributes={"message": msg})
+                if self.is_alive():
+                    self.log.error(
+                        "%s heartbeat failed with error. Scheduler may go into unhealthy state",
+                        self.__class__.__name__,
+                    )
+                    msg = f"{self.__class__.__name__} heartbeat failed with error. Scheduler may go into unhealthy state"
+                    if span.is_recording():
+                        span.add_event(name="error", attributes={"message": msg})
+                else:
+                    msg = f"{self.__class__.__name__} heartbeat failed with error. Scheduler is in unhealthy state"
+                    self.log.error(msg)
+                    if span.is_recording():
+                        span.add_event(name="error", attributes={"message": msg})
+                # We didn't manage to heartbeat, so make sure that the timestamp isn't updated
+                self.latest_heartbeat = previous_heartbeat
 
     @provide_session
     def prepare_for_execution(self, session: Session = NEW_SESSION):
@@ -278,22 +306,13 @@ class Job(Base, LoggingMixin):
 
     @staticmethod
     def _is_alive(
-        job_type: str | None,
-        heartrate: float,
         state: JobState | str | None,
+        health_check_threshold_value: float | int,
         latest_heartbeat: datetime.datetime,
-        grace_multiplier: float = 2.1,
     ) -> bool:
-        health_check_threshold: float
-        if job_type == "SchedulerJob":
-            health_check_threshold = conf.getint("scheduler", "scheduler_health_check_threshold")
-        elif job_type == "TriggererJob":
-            health_check_threshold = conf.getint("triggerer", "triggerer_health_check_threshold")
-        else:
-            health_check_threshold = heartrate * grace_multiplier
         return (
             state == JobState.RUNNING
-            and (timezone.utcnow() - latest_heartbeat).total_seconds() < health_check_threshold
+            and (timezone.utcnow() - latest_heartbeat).total_seconds() < health_check_threshold_value
         )
 
     @staticmethod
@@ -440,6 +459,7 @@ def execute_job(job: Job, execute_callable: Callable[[], int | None]) -> int | N
     return ret
 
 
+@span
 def perform_heartbeat(
     job: Job, heartbeat_callback: Callable[[Session], None], only_if_necessary: bool
 ) -> None:
