@@ -42,6 +42,7 @@ class PoolStats(TypedDict):
     deferred: int
     queued: int
     open: int
+    scheduled: int
 
 
 class Pool(Base):
@@ -178,11 +179,14 @@ class Pool(Base):
         for pool_name, total_slots, include_deferred in pool_rows:
             if total_slots == -1:
                 total_slots = float("inf")  # type: ignore
-            pools[pool_name] = PoolStats(total=total_slots, running=0, queued=0, open=0, deferred=0)
+            pools[pool_name] = PoolStats(
+                total=total_slots, running=0, queued=0, open=0, deferred=0, scheduled=0
+            )
             pool_includes_deferred[pool_name] = include_deferred
 
         allowed_execution_states = EXECUTION_STATES | {
             TaskInstanceState.DEFERRED,
+            TaskInstanceState.SCHEDULED,
         }
         state_count_by_pool = session.execute(
             select(TaskInstance.pool, TaskInstance.state, func.sum(TaskInstance.pool_slots))
@@ -205,6 +209,8 @@ class Pool(Base):
                 stats_dict["queued"] = count
             elif state == TaskInstanceState.DEFERRED:
                 stats_dict["deferred"] = count
+            elif state == TaskInstanceState.SCHEDULED:
+                stats_dict["scheduled"] = count
             else:
                 raise AirflowException(f"Unexpected state. Expected values: {allowed_execution_states}.")
 
