@@ -17,18 +17,14 @@
 # under the License.
 from __future__ import annotations
 
-import collections.abc
-import contextlib
 import inspect
-import itertools
 import json
 import logging
 import pickle
 import warnings
-from functools import cached_property, wraps
-from typing import TYPE_CHECKING, Any, Generator, Iterable, cast, overload
+from functools import wraps
+from typing import TYPE_CHECKING, Any, Iterable, cast, overload
 
-import attr
 from sqlalchemy import (
     Column,
     ForeignKeyConstraint,
@@ -38,6 +34,7 @@ from sqlalchemy import (
     PrimaryKeyConstraint,
     String,
     delete,
+    select,
     text,
 )
 from sqlalchemy.dialects.mysql import LONGBLOB
@@ -45,12 +42,12 @@ from sqlalchemy.ext.associationproxy import association_proxy
 from sqlalchemy.orm import Query, reconstructor, relationship
 from sqlalchemy.orm.exc import NoResultFound
 
-from airflow import settings
 from airflow.api_internal.internal_api_call import internal_api_call
 from airflow.configuration import conf
 from airflow.exceptions import RemovedInAirflow3Warning
 from airflow.models.base import COLLATION_ARGS, ID_LEN, TaskInstanceDependencies
 from airflow.utils import timezone
+from airflow.utils.db import LazySelectSequence
 from airflow.utils.helpers import exactly_one, is_container
 from airflow.utils.json import XComDecoder, XComEncoder
 from airflow.utils.log.logging_mixin import LoggingMixin
@@ -70,7 +67,9 @@ if TYPE_CHECKING:
     import datetime
 
     import pendulum
+    from sqlalchemy.engine import Row
     from sqlalchemy.orm import Session
+    from sqlalchemy.sql.expression import Select, TextClause
 
     from airflow.models.taskinstancekey import TaskInstanceKey
 
@@ -120,6 +119,7 @@ class BaseXCom(TaskInstanceDependencies, LoggingMixin):
         passive_deletes="all",
     )
     execution_date = association_proxy("dag_run", "execution_date")
+    logical_date = association_proxy("dag_run", "execution_date")
 
     @reconstructor
     def init_on_load(self):
@@ -148,7 +148,8 @@ class BaseXCom(TaskInstanceDependencies, LoggingMixin):
         map_index: int = -1,
         session: Session = NEW_SESSION,
     ) -> None:
-        """Store an XCom value.
+        """
+        Store an XCom value.
 
         A deprecated form of this function accepts ``execution_date`` instead of
         ``run_id``. The two arguments are mutually exclusive.
@@ -175,12 +176,14 @@ class BaseXCom(TaskInstanceDependencies, LoggingMixin):
         execution_date: datetime.datetime,
         session: Session = NEW_SESSION,
     ) -> None:
-        """Store an XCom value.
+        """
+        Store an XCom value.
 
         :sphinx-autoapi-skip:
         """
 
     @classmethod
+    @internal_api_call
     @provide_session
     def set(
         cls,
@@ -194,7 +197,8 @@ class BaseXCom(TaskInstanceDependencies, LoggingMixin):
         run_id: str | None = None,
         map_index: int = -1,
     ) -> None:
-        """Store an XCom value.
+        """
+        Store an XCom value.
 
         :sphinx-autoapi-skip:
         """
@@ -222,11 +226,11 @@ class BaseXCom(TaskInstanceDependencies, LoggingMixin):
             if dag_run_id is None:
                 raise ValueError(f"DAG run not found on DAG {dag_id!r} with ID {run_id!r}")
 
-        # Seamlessly resolve LazyXComAccess to a list. This is intended to work
+        # Seamlessly resolve LazySelectSequence to a list. This intends to work
         # as a "lazy list" to avoid pulling a ton of XComs unnecessarily, but if
         # it's pushed into XCom, the user should be aware of the performance
         # implications, and this avoids leaking the implementation detail.
-        if isinstance(value, LazyXComAccess):
+        if isinstance(value, LazySelectSequence):
             warning_message = (
                 "Coercing mapped lazy proxy %s from task %s (DAG %s, run %s) "
                 "to list, which may degrade performance. Review resource "
@@ -283,7 +287,8 @@ class BaseXCom(TaskInstanceDependencies, LoggingMixin):
         key: str | None = None,
         session: Session = NEW_SESSION,
     ) -> Any:
-        """Retrieve an XCom value for a task instance.
+        """
+        Retrieve an XCom value for a task instance.
 
         This method returns "full" XCom values (i.e. uses ``deserialize_value``
         from the XCom backend). Use :meth:`get_many` if you want the "shortened"
@@ -319,7 +324,8 @@ class BaseXCom(TaskInstanceDependencies, LoggingMixin):
         map_index: int | None = None,
         session: Session = NEW_SESSION,
     ) -> Any | None:
-        """Retrieve an XCom value, optionally meeting certain criteria.
+        """
+        Retrieve an XCom value, optionally meeting certain criteria.
 
         This method returns "full" XCom values (i.e. uses ``deserialize_value``
         from the XCom backend). Use :meth:`get_many` if you want the "shortened"
@@ -361,7 +367,8 @@ class BaseXCom(TaskInstanceDependencies, LoggingMixin):
         include_prior_dates: bool = False,
         session: Session = NEW_SESSION,
     ) -> Any | None:
-        """Retrieve an XCom value, optionally meeting certain criteria.
+        """
+        Retrieve an XCom value, optionally meeting certain criteria.
 
         :sphinx-autoapi-skip:
         """
@@ -380,7 +387,8 @@ class BaseXCom(TaskInstanceDependencies, LoggingMixin):
         run_id: str | None = None,
         map_index: int | None = None,
     ) -> Any | None:
-        """Retrieve an XCom value, optionally meeting certain criteria.
+        """
+        Retrieve an XCom value, optionally meeting certain criteria.
 
         :sphinx-autoapi-skip:
         """
@@ -435,7 +443,8 @@ class BaseXCom(TaskInstanceDependencies, LoggingMixin):
         limit: int | None = None,
         session: Session = NEW_SESSION,
     ) -> Query:
-        """Composes a query to get one or more XCom entries.
+        """
+        Composes a query to get one or more XCom entries.
 
         This function returns an SQLAlchemy query of full XCom objects. If you
         just want one stored value, use :meth:`get_one` instead.
@@ -473,14 +482,17 @@ class BaseXCom(TaskInstanceDependencies, LoggingMixin):
         limit: int | None = None,
         session: Session = NEW_SESSION,
     ) -> Query:
-        """Composes a query to get one or more XCom entries.
+        """
+        Composes a query to get one or more XCom entries.
 
         :sphinx-autoapi-skip:
         """
 
+    # The 'get_many` is not supported via database isolation mode. Attempting to use it in DB isolation
+    # mode will result in a crash - Resulting Query object cannot be **really** serialized
+    # TODO(potiuk) - document it in AIP-44 docs
     @staticmethod
     @provide_session
-    @internal_api_call
     def get_many(
         execution_date: datetime.datetime | None = None,
         key: str | None = None,
@@ -493,7 +505,8 @@ class BaseXCom(TaskInstanceDependencies, LoggingMixin):
         *,
         run_id: str | None = None,
     ) -> Query:
-        """Composes a query to get one or more XCom entries.
+        """
+        Composes a query to get one or more XCom entries.
 
         :sphinx-autoapi-skip:
         """
@@ -577,7 +590,8 @@ class BaseXCom(TaskInstanceDependencies, LoggingMixin):
         map_index: int | None = None,
         session: Session = NEW_SESSION,
     ) -> None:
-        """Clear all XCom data from the database for the given task instance.
+        """
+        Clear all XCom data from the database for the given task instance.
 
         A deprecated form of this function accepts ``execution_date`` instead of
         ``run_id``. The two arguments are mutually exclusive.
@@ -600,7 +614,8 @@ class BaseXCom(TaskInstanceDependencies, LoggingMixin):
         task_id: str,
         session: Session = NEW_SESSION,
     ) -> None:
-        """Clear all XCom data from the database for the given task instance.
+        """
+        Clear all XCom data from the database for the given task instance.
 
         :sphinx-autoapi-skip:
         """
@@ -617,7 +632,8 @@ class BaseXCom(TaskInstanceDependencies, LoggingMixin):
         run_id: str | None = None,
         map_index: int | None = None,
     ) -> None:
-        """Clear all XCom data from the database for the given task instance.
+        """
+        Clear all XCom data from the database for the given task instance.
 
         :sphinx-autoapi-skip:
         """
@@ -716,115 +732,25 @@ class BaseXCom(TaskInstanceDependencies, LoggingMixin):
         return BaseXCom._deserialize_value(self, True)
 
 
-class _LazyXComAccessIterator(collections.abc.Iterator):
-    def __init__(self, cm: contextlib.AbstractContextManager[Query]) -> None:
-        self._cm = cm
-        self._entered = False
-
-    def __del__(self) -> None:
-        if self._entered:
-            self._cm.__exit__(None, None, None)
-
-    def __iter__(self) -> collections.abc.Iterator:
-        return self
-
-    def __next__(self) -> Any:
-        return XCom.deserialize_value(next(self._it))
-
-    @cached_property
-    def _it(self) -> collections.abc.Iterator:
-        self._entered = True
-        return iter(self._cm.__enter__())
-
-
-@attr.define(slots=True)
-class LazyXComAccess(collections.abc.Sequence):
-    """Wrapper to lazily pull XCom with a sequence-like interface.
-
-    Note that since the session bound to the parent query may have died when we
-    actually access the sequence's content, we must create a new session
-    for every function call with ``with_session()``.
+class LazyXComSelectSequence(LazySelectSequence[Any]):
+    """
+    List-like interface to lazily access XCom values.
 
     :meta private:
     """
 
-    _query: Query
-    _len: int | None = attr.ib(init=False, default=None)
+    @staticmethod
+    def _rebuild_select(stmt: TextClause) -> Select:
+        return select(XCom.value).from_statement(stmt)
 
-    @classmethod
-    def build_from_xcom_query(cls, query: Query) -> LazyXComAccess:
-        return cls(query=query.with_entities(XCom.value))
-
-    def __repr__(self) -> str:
-        return f"LazyXComAccess([{len(self)} items])"
-
-    def __str__(self) -> str:
-        return str(list(self))
-
-    def __eq__(self, other: Any) -> bool:
-        if isinstance(other, (list, LazyXComAccess)):
-            z = itertools.zip_longest(iter(self), iter(other), fillvalue=object())
-            return all(x == y for x, y in z)
-        return NotImplemented
-
-    def __getstate__(self) -> Any:
-        # We don't want to go to the trouble of serializing the entire Query
-        # object, including its filters, hints, etc. (plus SQLAlchemy does not
-        # provide a public API to inspect a query's contents). Converting the
-        # query into a SQL string is the best we can get. Theoratically we can
-        # do the same for count(), but I think it should be performant enough to
-        # calculate only that eagerly.
-        with self._get_bound_query() as query:
-            statement = query.statement.compile(
-                query.session.get_bind(),
-                # This inlines all the values into the SQL string to simplify
-                # cross-process commuinication as much as possible.
-                compile_kwargs={"literal_binds": True},
-            )
-            return (str(statement), query.count())
-
-    def __setstate__(self, state: Any) -> None:
-        statement, self._len = state
-        self._query = Query(XCom.value).from_statement(text(statement))
-
-    def __len__(self):
-        if self._len is None:
-            with self._get_bound_query() as query:
-                self._len = query.count()
-        return self._len
-
-    def __iter__(self):
-        return _LazyXComAccessIterator(self._get_bound_query())
-
-    def __getitem__(self, key):
-        if not isinstance(key, int):
-            raise ValueError("only support index access for now")
-        try:
-            with self._get_bound_query() as query:
-                r = query.offset(key).limit(1).one()
-        except NoResultFound:
-            raise IndexError(key) from None
-        return XCom.deserialize_value(r)
-
-    @contextlib.contextmanager
-    def _get_bound_query(self) -> Generator[Query, None, None]:
-        # Do we have a valid session already?
-        if self._query.session and self._query.session.is_active:
-            yield self._query
-            return
-
-        Session = getattr(settings, "Session", None)
-        if Session is None:
-            raise RuntimeError("Session must be set before!")
-        session = Session()
-        try:
-            yield self._query.with_session(session)
-        finally:
-            session.close()
+    @staticmethod
+    def _process_row(row: Row) -> Any:
+        return XCom.deserialize_value(row)
 
 
 def _patch_outdated_serializer(clazz: type[BaseXCom], params: Iterable[str]) -> None:
-    """Patch a custom ``serialize_value`` to accept the modern signature.
+    """
+    Patch a custom ``serialize_value`` to accept the modern signature.
 
     To give custom XCom backends more flexibility with how they store values, we
     now forward all params passed to ``XCom.set`` to ``XCom.serialize_value``.
@@ -842,6 +768,7 @@ def _patch_outdated_serializer(clazz: type[BaseXCom], params: Iterable[str]) ->
             f"must be updated to accept all params in `BaseXCom.set` except `session`. Support will be "
             f"removed in a future release.",
             RemovedInAirflow3Warning,
+            stacklevel=1,
         )
         return old_serializer(**kwargs)
 
@@ -862,7 +789,8 @@ def _get_function_params(function) -> list[str]:
 
 
 def resolve_xcom_backend() -> type[BaseXCom]:
-    """Resolve custom XCom class.
+    """
+    Resolve custom XCom class.
 
     Confirm that custom XCom class extends the BaseXCom.
     Compare the function signature of the custom XCom serialize_value to the base XCom serialize_value.
