@@ -800,13 +800,6 @@ def initialize():
     atexit.register(dispose_orm)
 
 
-def is_usage_data_collection_enabled() -> bool:
-    """Check if data collection is enabled."""
-    return conf.getboolean("usage_data_collection", "enabled", fallback=True) and (
-        os.getenv("SCARF_ANALYTICS", "").strip().lower() != "false"
-    )
-
-
 # Const stuff
 
 KILOBYTE = 1024
